
package ca.ualberta.xdong.plasMapper.annotate;

import java.util.*;

/**
 * Defines a plasmid feature site. For example,there
 * are restriction site, promoter site.  Each site has start
 * position, end position, Label(name), and direction
 *
 *  @author xdong
 *
 */

public class FeatureSite {

	protected int start;
	protected int end;
	protected String label;
	
	//direction of the strand
	protected String direction = PlasMapperConstants.NODIRECTION + "";
	
	//1 forward
	//2 reverse;
	
	
	public FeatureSite(int start, int end, String label, String direction) {

		this.start = start;
		this.end = end;
		this.label = label;
		this.direction = direction;
	}

	public String getDirection() {

		return this.direction;
	}
	public int getStart() {

		return this.start;
	}

	public int getEnd() {

		return this.end;
	}

	public String getLabel() {

		return this.label;
	}

	public void setStart(int temp) {

		this.start = temp;
	}

	public void setEnd(int temp) {

		this.end = temp;
	}

	public void setLabel(String str) {

		this.label = str;
	}
	public String toString() {

		return "(" + start + "," + end + "," + label + ")";

	}
	
	/**
	 * Sort the Vector contains FeatureSite object accroding the start
	 * position of the site object. 
	 * 
	 * @param locations	a vector of the FeatureSite objects
	 * @param size	
	 * @return	a vector of FeatureSite object that is sorted by
	 * using start positon 
	 */
	/*
	  sort the given vector according start position of cloningsite
	*/
	public static Vector bubbleSort(Vector locations, int size) {

		int i, j;

		FeatureSite locationI = null;
		FeatureSite locationJ = null;

		for (i = 0; i < size - 1; i++) {

			for (j = i + 1; j < size; j++) {
				locationI = (FeatureSite) locations.elementAt(i);
				locationJ = (FeatureSite) locations.elementAt(j);
				//if (array[i]>array[j])
				if (locationI.getStart() > locationJ.getStart()) { // SWAP
					
					locations.setElementAt(locationJ, i);
					locations.setElementAt(locationI, j);
				}
			}
		}

		return locations;
	}
	
	public boolean equals(FeatureSite site)
		{
		String name = site.getLabel();
	
		if((this.label.equals(name)) && (this.start == site.getStart()) && (this.end == site.getEnd()))
			return true;
		else 
			return false;
		}
}
