package ca.ualberta.xdong.plasMapper.annotate;
import org.biojava.bio.*;
import org.biojava.bio.seq.*;
import org.biojava.bio.symbol.*;
import org.biojava.bio.molbio.RestrictionEnzyme;
import org.biojava.utils.SimpleThreadPool;
import org.biojava.bio.molbio.RestrictionMapper;
import org.biojava.bio.molbio.*;
import java.util.*;

/**
 * 
 * @author xdong
 */

public class PlasmidCleavageSiteAnnotate {

	protected String sequence;
	protected Vector commonSet;
	protected Vector enzymeNames;

	String[] commonEnzymeName =
		{
			"HpaII",
			"ApoI",
			"SacI",
			"EcoRI",
			"AlwNI",
			"AluI",
			"AccI",
			"MboI",
			"Sau3AI",
			"SfiI",
			"HincII",
			"NruI",
			"BglI",
			"AvrI",
			"XmaI",
			"DraI",
			"PvuII",
			"HindIII",
			"KpnI",
			"NarI",
			"BglII",
			"HpaI",
			"StuI",
			"NcoI",
			"NdeI",
			"Bsp24I",
			"BclI",
			"SmaI",
			"AvaI",
			"AvaII",
			"PstI",
			"SphI",
			"ApaI",
			"EcoRV",
			"BamHI",
			"HaeIII",
			"NotI",
			"XhoI",
			"ClaI",
			"XbaI" };

	public PlasmidCleavageSiteAnnotate(String sequence) {

		this.sequence = sequence;
		this.commonSet = new Vector(40);

		for (int i = 0; i < commonEnzymeName.length; i++) {
			commonSet.add(commonEnzymeName[i]);
		}

		this.enzymeNames = getAllEnzymeNames();

	}

	public String getSeq() {

		return this.sequence;
	}

	public int getSeqLength() {

		return this.sequence.length();
	}
	
	/**
	 * 
	 * @return all the enzyme Names that restrictionManager uses
	 */
	public Vector getAllEnzymeNames() {

		int i = 0;
		Vector enzymeNames = new Vector(20);

		//get all the enzymes
		Set set = RestrictionEnzymeManager.getAllEnzymes();

		StringTokenizer st = null;
		String str = null;
		Iterator ii = set.iterator();

		int counter = 0;
		while (ii.hasNext()) {

			//string: name pattern (n/m)
			str = (ii.next()).toString();
			//System.out.println(str);
			st = new StringTokenizer(str);
			i = 0;
			String temp = "";

			while (st.hasMoreTokens()) {
				String strr = st.nextToken();
				temp = temp + " " + strr;
				//enzymeName is the first token
				if (i == 0) {
					counter++;
					enzymeNames.addElement(strr);
					i++;
				}
			}
		}
		//System.out.println("enzyme number=" + counter);
		return enzymeNames;
	}


	/**
	 * Use all the enzyme the given enzyme database to anot
	 * 
	 * ate the 
	 * sequence. Then get a collection of all the restriction sites
	 * (FeatureSite). This will not remove the duplication of the enzyme  
	 * 
	 * @param enzymeType All the enzyme set or common enzyme set
	 * @return	a collection of the FeatureSite objects which represents 
	 * all the restriction sites
	 * @throws Exception
	 */
	public Vector getRestrictionSites(int enzymeType) throws Exception {
		
		Sequence seq = DNATools.createDNASequence(this.sequence, "dna");

		//create RestrictionMapper
		SimpleThreadPool threadPool = new SimpleThreadPool();
		RestrictionMapper mapper = new RestrictionMapper(threadPool);

		//when annotate sequence, only enzymies that added to maper will be considered
		//Add all the enzyme in RestrictioEnzymeManager into mapper
		if (enzymeType == PlasMapperConstants.ALL_ENZYME) {
			for (int i = 0; i < this.enzymeNames.size(); i++) {
				String enzymeName = (String) enzymeNames.elementAt(i);
				//System.out.println(enzymeName);
				RestrictionEnzyme enzyme =
					RestrictionEnzymeManager.getEnzyme(enzymeName);
				mapper.addEnzyme(enzyme);
			}
		}

		//common enzyme set
		else if (enzymeType == PlasMapperConstants.COMMON_ENZYME ) {

			for (int i = 0; i < commonSet.size(); i++) {
				String enzymeName = (String) commonSet.elementAt(i);
				//System.out.println(enzymeName);
				RestrictionEnzyme enzyme =
					RestrictionEnzymeManager.getEnzyme(enzymeName);
				mapper.addEnzyme(enzyme);
			}

		}

		//annotate the seq by using the enzymes in the RestrictionMapper
		Sequence seqAn = mapper.annotate(seq);

		//get the feature for the annotated seq
		Iterator fi = seqAn.features();
		Feature f = null;
		SymbolList symbol = null;
		String pattern = null;
		Annotation fAn = null;

		String name = "";
		int start = 0;
		FeatureSite site = null;
		Vector siteVector = new Vector(20);
		while (fi.hasNext()) {

			f = (Feature) fi.next();

			fAn = f.getAnnotation();
			//System.out.println((fAn.getProperty("dbxref")).toString());
			name = ((fAn.getProperty("dbxref")).toString()).substring(7);

			int downPosition =
				(RestrictionEnzymeManager
					.getEnzyme(name)
					.getDownstreamCut())[0];
			//get recognization sequence
			symbol = f.getSymbols();
			pattern = symbol.seqString().toUpperCase();

			//getRange of the enzyme
			start = (f.getLocation()).getMin();
			//end = (f.getLocation()).getMax();
			int cutPos = start + downPosition;

			site = new FeatureSite(cutPos, cutPos, name, pattern);
			siteVector.add(site);

		}

		//sort all restrictionSites
		siteVector = FeatureSite.bubbleSort(siteVector, siteVector.size());

		return siteVector;
	}
	/**
		 * Use all the enzyme the given enzyme database to anot
		 * 
		 * ate the 
		 * sequence. Then get a collection of all the restriction sites
		 * (FeatureSite). This will not remove the duplication of the enzyme  
		 * 
		 * @param enzymeType All the enzyme set or common enzyme set
		 * @return	a collection of the FeatureSite objects which represents 
		 * all the restriction sites
		 * @throws Exception
		 */
		public Vector getRestrictionSitesWithStartEndPos(int enzymeType) throws Exception {
		
			Sequence seq = DNATools.createDNASequence(this.sequence, "dna");

			//create RestrictionMapper
			SimpleThreadPool threadPool = new SimpleThreadPool();
			RestrictionMapper mapper = new RestrictionMapper(threadPool);

			//when annotate sequence, only enzymies that added to maper will be considered
			//Add all the enzyme in RestrictioEnzymeManager into mapper
			if (enzymeType == PlasMapperConstants.ALL_ENZYME) {
				for (int i = 0; i < this.enzymeNames.size(); i++) {
					String enzymeName = (String) enzymeNames.elementAt(i);
					//System.out.println(enzymeName);
					RestrictionEnzyme enzyme =
						RestrictionEnzymeManager.getEnzyme(enzymeName);
					mapper.addEnzyme(enzyme);
				}
			}

			//common enzyme set
			else if (enzymeType == PlasMapperConstants.COMMON_ENZYME ) {

				for (int i = 0; i < commonSet.size(); i++) {
					String enzymeName = (String) commonSet.elementAt(i);
					//System.out.println(enzymeName);
					RestrictionEnzyme enzyme =
						RestrictionEnzymeManager.getEnzyme(enzymeName);
					mapper.addEnzyme(enzyme);
				}

			}

			//annotate the seq by using the enzymes in the RestrictionMapper
			Sequence seqAn = mapper.annotate(seq);

			//get the feature for the annotated seq
			Iterator fi = seqAn.features();
			Feature f = null;
			SymbolList symbol = null;
			String pattern = null;
			Annotation fAn = null;

			String name = "";
			int start = 0;
			int end = 0;
			FeatureSite site = null;
			Vector siteVector = new Vector(20);
			while (fi.hasNext()) {

				f = (Feature) fi.next();

				fAn = f.getAnnotation();
				//System.out.println((fAn.getProperty("dbxref")).toString());
				name = ((fAn.getProperty("dbxref")).toString()).substring(7);

				
				//get recognization sequence
				symbol = f.getSymbols();
				pattern = symbol.seqString().toUpperCase();

				//getRange of the enzyme
				start = (f.getLocation()).getMin();
				end = (f.getLocation()).getMax();

				site = new FeatureSite(start, end, name, pattern);
				siteVector.add(site);

			}

			//sort all restrictionSites
			siteVector = FeatureSite.bubbleSort(siteVector, siteVector.size());

			return siteVector;
		}

	
	/**
	 * Use certain cutter in the given enzyme database to anotate the 
	 * sequence. Then get a collection of all the restriction sites
	 * (FeatureSite). This will not remove the duplication of the enzyme  
	 * 
	 * @param nCutters	a collection of the cutters
	 * @param enzymeType All the enzyme set or common enzyme set
	 * @return	a collection of the FeatureSite objects which represents 
	 * all the restriction sites
	 * @throws Exception
	 */
	
	public Vector getRestrictionSites(Vector nCutters, int enzymeType)
		throws Exception {
		//Vector enzymeNames = new Vector(20);
		//enzymeNames = getAllEnzymeNames();

		//DNATools dna = new DNATools();
		Sequence seq = DNATools.createDNASequence(this.sequence, "dna");

		//create RestrictionMapper
		SimpleThreadPool threadPool = new SimpleThreadPool();
		RestrictionMapper mapper = new RestrictionMapper(threadPool);

		//when annotate sequence, only enzymies that added to maper will be considered

		for (int i = 0; i < nCutters.size(); i++) {

			Integer nCutter = (Integer) nCutters.elementAt(i);
			Set nCutterEnzymes =
				RestrictionEnzymeManager.getNCutters(nCutter.intValue());

			for (Iterator ei = nCutterEnzymes.iterator(); ei.hasNext();) {
				RestrictionEnzyme re = (RestrictionEnzyme) ei.next();
				//System.out.println(re.getName());

				if (enzymeType == PlasMapperConstants.ALL_ENZYME )
					mapper.addEnzyme(re);
				else if (
					enzymeType == PlasMapperConstants.COMMON_ENZYME && findElement(commonSet, re.getName()))
					mapper.addEnzyme(re);
			}
		}

		//annotate the seq by using the enzymes in the RestrictionMapper
		Sequence seqAn = mapper.annotate(seq);

		//get the feature for the annotated seq
		Iterator fi = seqAn.features();
		Feature f = null;
		SymbolList symbol = null;
		String pattern = null;
		Annotation fAn = null;

		String name = "";
		int start = 0;
		FeatureSite site = null;
		Vector siteVector = new Vector(20);
		while (fi.hasNext()) {

			f = (Feature) fi.next();
			fAn = f.getAnnotation();
			name = ((fAn.getProperty("dbxref")).toString()).substring(7);

			//System.out.println(RestrictionEnzymeManager.getEnzyme(name));
			int downPosition =
				(RestrictionEnzymeManager
					.getEnzyme(name)
					.getDownstreamCut())[0];
			//System.out.println(fAn.getProperty("dbxref"));
			//get recognization sequence
			symbol = f.getSymbols();
			pattern = symbol.seqString().toUpperCase();

			//getRange of the enzyme
			start = (f.getLocation()).getMin();
			//end = (f.getLocation()).getMax();
			int cutPos = start + downPosition;

			site = new FeatureSite(cutPos, cutPos, name, pattern);

			siteVector.add(site);

		}

		//sort all restrictionSites
		siteVector = FeatureSite.bubbleSort(siteVector, siteVector.size());

		return siteVector;
	}
	
	/**
		 * Use certain cutter in the given enzyme database to anotate the 
		 * sequence. Then get a collection of all the restriction sites
		 * (FeatureSite). This will not remove the duplication of the enzyme  
		 * 
		 * @param nCutters	a collection of the cutters
		 * @param enzymeType All the enzyme set or common enzyme set
		 * @return	a collection of the FeatureSite objects which represents 
		 * all the restriction sites
		 * @throws Exception
		 */
	
		public Vector getRestrictionSitesWithStartEndPos(Vector nCutters, int enzymeType)
			throws Exception {
			//Vector enzymeNames = new Vector(20);
			//enzymeNames = getAllEnzymeNames();

			//DNATools dna = new DNATools();
			Sequence seq = DNATools.createDNASequence(this.sequence, "dna");

			//create RestrictionMapper
			SimpleThreadPool threadPool = new SimpleThreadPool();
			RestrictionMapper mapper = new RestrictionMapper(threadPool);

			//when annotate sequence, only enzymies that added to maper will be considered

			for (int i = 0; i < nCutters.size(); i++) {

				Integer nCutter = (Integer) nCutters.elementAt(i);
				Set nCutterEnzymes =
					RestrictionEnzymeManager.getNCutters(nCutter.intValue());

				for (Iterator ei = nCutterEnzymes.iterator(); ei.hasNext();) {
					RestrictionEnzyme re = (RestrictionEnzyme) ei.next();
					//System.out.println(re.getName());

					if (enzymeType == PlasMapperConstants.ALL_ENZYME )
						mapper.addEnzyme(re);
					else if (
						enzymeType == PlasMapperConstants.COMMON_ENZYME && findElement(commonSet, re.getName()))
						mapper.addEnzyme(re);
				}
			}

			//annotate the seq by using the enzymes in the RestrictionMapper
			Sequence seqAn = mapper.annotate(seq);

			//get the feature for the annotated seq
			Iterator fi = seqAn.features();
			Feature f = null;
			SymbolList symbol = null;
			String pattern = null;
			Annotation fAn = null;

			String name = "";
			int start = 0;
			int end = 0;
			FeatureSite site = null;
			Vector siteVector = new Vector(20);
			while (fi.hasNext()) {

				f = (Feature) fi.next();

				fAn = f.getAnnotation();
				name = ((fAn.getProperty("dbxref")).toString()).substring(7);

				//System.out.println(RestrictionEnzymeManager.getEnzyme(name));

				
				//System.out.println(fAn.getProperty("dbxref"));
				//get recognization sequence
				symbol = f.getSymbols();
				pattern = symbol.seqString().toUpperCase();

				//getRange of the enzyme
				start = (f.getLocation()).getMin();
				end = (f.getLocation()).getMax();

				site = new FeatureSite(start, end, name, pattern);

				siteVector.add(site);

			}

			//sort all restrictionSites
			siteVector = FeatureSite.bubbleSort(siteVector, siteVector.size());

			return siteVector;
		}
	/**
	 * Remove duplicated restriction sites for the give siteVector. 
	 * keep the unique restriction site
	 * 
	 * @param siteVector
	 * @return	a collection of FeatureSite objects which represent
	 * all the unique restriction sites
	 */
	
	public Vector getUniqueRestrictionSites(Vector siteVector) {

		Vector uniqueVec = new Vector(10);

		//get unique cutters
		for (int i = 0; i < siteVector.size(); i++) {
			FeatureSite site = (FeatureSite) siteVector.elementAt(i);

			if (count(site, siteVector) == 1)
				uniqueVec.add(site);
		}

		return uniqueVec;
	}

	
	/**
	 * Get the total numbers of the given FeatureSite object 
	 * in the given  FeatureSite vector
	 * 
	 * @param site	a given fearture
	 * @param siteVector	
	 * @return total number of the site object in siteVector
	 */

	private int count(FeatureSite site, Vector siteVector) {

		FeatureSite tmpSite = null;
		int count = 0;

		for (int i = 0; i < siteVector.size(); i++) {
			tmpSite = (FeatureSite) siteVector.elementAt(i);
			if ((site.getLabel()).equals(tmpSite.getLabel()))
				count++;

		}
		return count;
	}

	/**
	 * Look for a str element in the given vector
	 * @param aVector
	 * @param str
	 * @return
	 */
	private boolean findElement(Vector aVector, String str) {
		int i;
		for (i = 0; i < aVector.size(); i++) {
			String aStr = (String) aVector.elementAt(i);
			if (str.equals(aStr))
				break;
		}
		if (i >= aVector.size())
			return false;
		return true;
	}
	/*
	public static void main(String[] args) throws Exception
	{
	  //readFile from a file
	  StringBuffer buffer = Utility.readString("/cbrmed01/uahome/xdong/jakarta-tomcat-4.1.24-LE-jdk14/webapps/labm/view/input.txt");
	
	  String sequence = new String(buffer);
	  PlasmidCleavageSiteAnnotate  site = new PlasmidCleavageSiteAnnotate(sequence);
	
	  Vector nCutters = new Vector(2);
	  //nCutters.add(new Integer(4));
	  nCutters.add(new Integer(6));
	  nCutters.add(new Integer(8));
	
	Vector siteVector = site.getRestrictionSites(nCutters);
	Vector uniqueVec = site.getUniqueRestrictionSites(siteVector);
	
	//all the enzyme
	Vector clusterVector = site.getClustedVector(siteVector,10);
	System.out.println(site.getClustedSites(clusterVector));
	
	System.out.println("This following is the unique cutter");
	//unique
	clusterVector = site.getClustedVector(uniqueVec,10);
	System.out.println(site.getClustedSites(clusterVector));
	
	
	//get mcs using unique enzyme
	Vector mcsVec = site.getMCS_vector(uniqueVec,15, 15);
	System.out.println("the number of MCS when consider just unique enzyme===" + mcsVec.size());
	System.out.println(site.getClustedSites(mcsVec));
	
	
	//get mcs using all enzyme
	mcsVec = site.getMCS_vector(siteVector,10, 15);
	System.out.println("the number of MCS when consider all enzyme===" + mcsVec.size());
	System.out.println(site.getClustedSites(mcsVec));
	
	
	
	
	}
	*/

}
