package ca.ualberta.xdong.plasMapper.annotate;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * 
 * @author xdong
 *
 */
public class PlasmidFeatureAnnotate {

	protected String inputFile;

	protected String outputFile;

	protected String program;

	protected String database;

	protected String htm;

	/**
	 * 
	 * @param inputFile
	 * @param outputFile
	 * @throws Exception
	 */
	public PlasmidFeatureAnnotate(String inputFile, String outputFile)
			throws Exception {

		Utility util = new Utility();
		this.inputFile = inputFile;
		this.outputFile = outputFile;
		this.program = "blastn";
		this.database = util.getPath("vectorFeature_db");
		this.htm = "F";
		
		doBlast();
	}

	/**
	 * Core function of this class.
	 * <p>
	 * Firstly check to see whether alignment length is equal
	 * to target length or not. If equals, it's possible to be exact macth. 
	 * Then check to see the identity. if identity>95% and length equals, 
	 * pick this target and put it into feature vector. The output file format 
	 * in every row looks like: QUERY SV40_large_T_antigen(aa_84_708)[OTH](subject)
	 * 1869(subject length) 100.00(identity) 13(alignment length)0(mismatches)
	 * 0(gap openings) 3873(query start) 3885(query end) 1593(subject start) 
	 * 1581(subject end) 9.9(e-value) 26.26(bit score)
	 * 
	 * @return a collection of the FeatureSite in the sequence
	 */
	
	public Vector getHitFeatures() {

		String str;
		StringTokenizer stk;
		Vector features = new Vector(5);

		try {

			BufferedReader br = new BufferedReader(new FileReader(
					this.outputFile));
			int i = 0;
			double identity = 0;
			int length = 0;
			int alignLength = 0;
			int countTokens = 0;
			Vector tokens;

			while ((str = br.readLine()) != null) {
				stk = new StringTokenizer(str, ",\t\n\r\f");
				i = 0;
				countTokens = 0;
				tokens = new Vector(countTokens);
				while (stk.hasMoreTokens()) {
					str = stk.nextToken();
					tokens.addElement(str);
					// System.out.println("this is string=" + str);
					i++;
				}

				// get the length of the current sequence in the database
				str = (String) tokens.elementAt(2);
				// System.out.println("string is:=" + str);
				length = Integer.parseInt(str);

				// check the alignment length
				str = (String) tokens.elementAt(4);
				alignLength = Integer.parseInt(str);

				// check the identities of the hitable
				str = (String) tokens.elementAt(3);
				identity = Double.valueOf(str).doubleValue();

				int direction = PlasMapperConstants.NODIRECTION;
				
				int subjectStart = Integer.parseInt((String) tokens
						.elementAt(9));
				int subjectEnd = Integer
						.parseInt((String) tokens.elementAt(10));

				// forward
				if (subjectStart < subjectEnd)
					direction = PlasMapperConstants.FORWARD;
				// reverse
				else if (subjectStart > subjectEnd)
					direction = PlasMapperConstants.REVERSE;

				String temp = (String) tokens.elementAt(1);
				// System.out.println("length*0.90=" + length*0.90);
				// System.out.println("length*0.95=" + length*0.95);
				// System.out.println("length*0.95=" + length);

				FeatureSite site = null;
				int startPos = -1, endPos = -1;
				// exact match
				if (length <= 100) {
					if (alignLength == length && identity >= 95) {
						// feature = temp + "," + (String)tokens.elementAt(7) +
						// "-"
						// + (String)tokens.elementAt(8) + "," + direction;
						startPos = Integer.parseInt((String) tokens
								.elementAt(7));
						endPos = Integer.parseInt((String) tokens.elementAt(8));

					}

				} else if (100 < length && length <= 500) {
					if (alignLength > (int) (length * 0.95) && identity >= 95) {
						// feature = temp + "," + (String)tokens.elementAt(7) +
						// "-"
						// + (String)tokens.elementAt(8) + "," + direction;

						startPos = Integer.parseInt((String) tokens
								.elementAt(7));
						endPos = Integer.parseInt((String) tokens.elementAt(8));
					}
				}

				else {

					if (alignLength > (int) (length * 0.90) && identity >= 95) {
						// feature = temp + "," + (String)tokens.elementAt(7) +
						// "-"
						// + (String)tokens.elementAt(8) + "," + direction;
						startPos = Integer.parseInt((String) tokens
								.elementAt(7));
						endPos = Integer.parseInt((String) tokens.elementAt(8));
					}
				}
				String strand = direction + "";

				if (startPos != -1 && endPos != -1) {
					site = new FeatureSite(startPos, endPos, temp, strand);
					features.addElement(site);
				}
				// add new antiobiotic to vector;
				// if(feature.length() != 0)
				// features.addElement(feature);
			}
			br.close();

		} catch (FileNotFoundException fnfe) {
			System.out.println(fnfe);

		} catch (IOException ioe) {
			System.out.println(ioe);
		}
		// System.out.println("feature=" + features);

		return features;
	}

	/**
	 * Invoke the blast search program
	 * 
	 * @throws InterruptedException
	 */
	protected void doBlast() throws InterruptedException {
		try {
			Utility util = new Utility();
			String blastPath = util.getPath("blastallDir");
			String blastCmd = blastPath + "blastall -p " + this.program
					+ " -d " + database + " -i " + this.inputFile + " -o "
					+ this.outputFile + " -m 8 -F F -T " + htm;
			Process p = Runtime.getRuntime().exec(blastCmd);
			p.waitFor();
		} catch (Exception e) {
			System.out.println(e);

		}
	}

}
