package ca.ualberta.xdong.plasMapper.annotate;

import java.util.*;
import java.io.*;
import java.sql.*;


/**
 *
 * @author xdong
 */

public class VectorSeqDatabase {

        private Connection dbcon;
        private PreparedStatement getNameIDStmt;
        private PreparedStatement getSeqStmt;
        private PreparedStatement putStmt;
        private PreparedStatement getVendorStmt;
        private String vendor;

        public VectorSeqDatabase() throws Exception {

                init();
        }

        public Connection getConnection() {
                return dbcon;
        }

        private void init() throws Exception {
                Utility util = new Utility();
                String db_user = util.getPath("db_user");
                String db_passwd = util.getPath("db_passwd");
                String db_url = util.getPath("db_url");
                String db_driver = util.getPath("db_driver");

                String put = "insert into vector values(?, ?,?,?)";
                String getNameID =
                        "select id,name from vector where vendor=? order by id";
                String getSeq = "select sequence from vector where id = ?";
                String getVendor = "select vendor from vector where id = ?";

                try {

                        Class.forName(db_driver).newInstance();

                        //Class.forName("com.mysql.jdbc.Driver").newInstance();
                        dbcon = DriverManager.getConnection(db_url, db_user, db_passwd);

                        getNameIDStmt = dbcon.prepareStatement(getNameID);
                        getSeqStmt = dbcon.prepareStatement(getSeq);
                        putStmt = dbcon.prepareStatement(put);
                        getVendorStmt = dbcon.prepareStatement(getVendor);

                } catch (SQLException ex) {

                        ex.printStackTrace();
                } // end catch SQLException

                catch (java.lang.Exception ex) {
                        ex.printStackTrace();
                        ex.getMessage();
                }
        }

        public void setVendor(String vendor) {
                this.vendor = vendor;
        }
        public String getVendor(String id) {

                ResultSet result = null;
                String vendor = null;
                try {
                        getVendorStmt.clearParameters();
                        getVendorStmt.setString(1, id);
                        result = getVendorStmt.executeQuery();
                        if (result.next()) {
                                vendor = result.getString("vendor");

                        }

                } catch (SQLException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                }
                return vendor;

        }
        public void putSeq(String id, String name, String seq) throws SQLException {
                try {

                        putStmt.clearParameters();
                        putStmt.setString(1, id);
                        putStmt.setString(2, name);
                        putStmt.setString(3, vendor);
                        putStmt.setString(4, seq);
                        putStmt.executeUpdate();

                } catch (SQLException ex) {
                        dbcon.rollback();
                        ex.printStackTrace();
                } catch (java.lang.Exception ex) {
                        ex.printStackTrace();
                        ex.getMessage();
                }
        }

        public Vector getAllNameID() throws Exception {
                // We cannot use generics, the java version is too old --pjs
		// Vector<LibrarySeq> libraries = new Vector<LibrarySeq>(100);
                Vector libraries = new Vector(100);
                try {
                        ResultSet result;
                        getNameIDStmt.clearParameters();
                        getNameIDStmt.setString(1, this.vendor);
                        result = getNameIDStmt.executeQuery();
                        LibrarySeq library = null;

                        while (result.next()) {
                                String tempID = result.getString("id");
                                String name = result.getString("name");

                                library = new LibrarySeq(tempID, name, "");
                                libraries.add(library);
                        }

                } catch (SQLException ex) {
                        dbcon.rollback();
                        ex.printStackTrace();
                } catch (java.lang.Exception ex) {
                        ex.printStackTrace();
                        ex.getMessage();
                }

                return libraries;
        }

        public StringBuffer getSeq(String id) throws Exception {

                //String seq = "";
                StringBuffer buffer = new StringBuffer(10000);

                try {

                        ResultSet result;
                        getSeqStmt.clearParameters();
                        getSeqStmt.setString(1, id);
                        result = getSeqStmt.executeQuery();
                        BufferedReader br = null;
                        InputStream in = null;

                        if (result.next()) {
                                in = result.getBinaryStream("sequence");
                                br = new BufferedReader(new InputStreamReader(in));
                        }

                        String str;
                        while ((str = br.readLine()) != null) {
                                //System.out.println(str);
                                buffer.append(str);
                        }
                        in.close();
                        br.close();

                } catch (SQLException ex) {
                        dbcon.rollback();
                        ex.printStackTrace();
                } catch (java.lang.Exception ex) {
                        ex.printStackTrace();
                        ex.getMessage();
                }

                return buffer;
        }

        //read String from a file
        public void readString(String file) throws IOException, SQLException {

                //readFile from a file
                String str;
                dbcon.setAutoCommit(false);
                String name = "";
                String seq = "";

                BufferedReader br = new BufferedReader(new FileReader(file));
                int i = 0;

                while ((str = br.readLine()) != null) {
                        //System.out.println("string=" + str );

                        if (str.indexOf(">") >= 0) {
                                //System.out.println("name=" + name);

                                if (name == null || name.length() == 0) {
                                        System.out.println(i);
                                        name = str;
                                        i++;
                                } else {
                                        if (seq != null
                                                && seq.length() > 0
                                                && seq.length() <= 20000) {
                                                String id = this.vendor + i;
                                                putSeq(id, name, seq);
                                        }

                                        name = str;
                                        i++;
                                        seq = "";
                                }
                        }
                        else
                                seq = seq + str;
                }

                if (i != 0)
                        putSeq(this.vendor + i, name, seq);
                dbcon.commit();

                br.close();
        }

        public void destroy() {

                if (dbcon != null) {

                        try {
                                getNameIDStmt.close();
                                getSeqStmt.close();
                                putStmt.close();

                                System.out.println(" destroy before close");
                                dbcon.close();

                                System.out.println("afer close destroy");
                        } catch (Exception e) {
                                ;
                        }

                }
        }
        public static void main(String[] args) throws Exception {
                System.out.println("start");

                VectorSeqDatabase test = new VectorSeqDatabase();
                System.out.println("before insert");
                Utility util = new Utility();
                String libRoot = util.getPath("lib_dir");

                test.setVendor("Amersham Pharmacia");
                test.readString(libRoot + "all_amersham_pharmacia");

                test.setVendor("Clontech");
                test.readString(libRoot + "all_clontech");

                test.setVendor("Epicentre");
                test.readString(libRoot + "all_epicentre");

                test.setVendor("IBI");
                test.readString(libRoot + "all_ibi");

                test.setVendor("Invitrogen");
                test.readString(libRoot + "all_invitrogen");

                test.setVendor("NEB");
                test.readString(libRoot + "all_neb");

                test.setVendor("Promega");
                test.readString(libRoot + "all_promega");

                test.setVendor("Public");
                test.readString(libRoot + "all_public");

                test.setVendor("Stratagene");
                test.readString(libRoot + "all_stratagene");

                test.setVendor("Lucigen");
                test.readString(libRoot + "all_lucigen");

                test.destroy();

                System.out.println("end");
        }
}
